@@ -71,6 +71,7 @@ from .idsjsonmarketlookup import IdsJsonMarketLookup as IdsJsonMarketLookup
 from .inventory import Inventory as Inventory
 from .inventory import PerpInventoryAccountWatcher as PerpInventoryAccountWatcher
 from .inventory import SpotInventoryAccountWatcher as SpotInventoryAccountWatcher
+from .instructions import build_cancel_all_perp_orders_instructions as build_cancel_all_perp_orders_instructions
 from .instructions import build_cancel_perp_order_instructions as build_cancel_perp_order_instructions
 from .instructions import build_cancel_spot_order_instructions as build_cancel_spot_order_instructions
 from .instructions import build_close_spl_account_instructions as build_close_spl_account_instructions
@@ -87,8 +88,10 @@ from .instructions import build_redeem_accrued_mango_instructions as build_redee
 from .instructions import build_serum_consume_events_instructions as build_serum_consume_events_instructions
 from .instructions import build_serum_place_order_instructions as build_serum_place_order_instructions
 from .instructions import build_serum_settle_instructions as build_serum_settle_instructions
+from .instructions import build_set_account_delegate_instructions as build_set_account_delegate_instructions
 from .instructions import build_spot_place_order_instructions as build_spot_place_order_instructions
 from .instructions import build_transfer_spl_tokens_instructions as build_transfer_spl_tokens_instructions
+from .instructions import build_unset_account_delegate_instructions as build_unset_account_delegate_instructions
 from .instructions import build_withdraw_instructions as build_withdraw_instructions
 from .instructionreporter import InstructionReporter as InstructionReporter
 from .instructionreporter import SerumInstructionReporter as SerumInstructionReporter
