@@ -21,8 +21,9 @@ import time
 import typing
 
 
-from base64 import b64decode, b64encode
+from base64 import b64decode
 from collections.abc import Mapping
+from concurrent.futures import Executor, ThreadPoolExecutor
 from decimal import Decimal
 from solana.blockhash import Blockhash, BlockhashCache
 from solana.keypair import Keypair
@@ -57,6 +58,9 @@ class CompoundException(Exception):
         self.all_exceptions: typing.Sequence[Exception] = all_exceptions
 
     def __str__(self) -> str:
+        if len(self.all_exceptions) == 1:
+            return str(self.all_exceptions[0])
+
         details: str = indent_collection_as_str(self.all_exceptions)
         return f"""« CompoundException with {len(self.all_exceptions)} inner exceptions:
     {details}
@@ -285,6 +289,58 @@ class SlotHolder:
         return True
 
 
+class TransactionWatcher:
+    def __init__(self, client: Client, slot_holder: SlotHolder, signature: str):
+        self._logger: logging.Logger = logging.getLogger(self.__class__.__name__)
+        self.client: Client = client
+        self.slot_holder: SlotHolder = slot_holder
+        self.signature: str = signature
+
+    def report_on_transaction(self) -> None:
+        started_at: float = time.time()
+        for pause in [0.1, 0.2, 0.3, 0.4, 0.5, 0.5, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]:
+            transaction_response = self.client.get_signature_statuses([self.signature])
+            if "result" in transaction_response and "value" in transaction_response["result"]:
+                [status] = transaction_response["result"]["value"]
+                if status is not None:
+                    time_taken: float = time.time() - started_at
+
+                    # value should be a dict that looks like:
+                    # {
+                    #   'confirmationStatus': 'processed',
+                    #   'confirmations': 0,
+                    #   'err': None,
+                    #   'slot': 116230143,
+                    #   'status': {'Ok': None}
+                    # }
+                    # If there's an error it should look like:
+                    # {
+                    #     'slot': 116235922,
+                    #     'confirmations': 0,
+                    #     'status': {'Err': {'InstructionError': [0, {'Custom': 24}]}},
+                    #     'err': {
+                    #         'InstructionError': [0, {'Custom': 24}]
+                    #     },
+                    #     'confirmationStatus': 'processed'
+                    # }
+                    if status["err"] is not None:
+                        self._logger.warning(
+                            f"Transaction {self.signature} failed after {time_taken} seconds with error {status['err']}")
+                        return
+
+                    confirmation_status: str = status["confirmationStatus"]
+                    slot: int = status["slot"]
+                    self.slot_holder.require_data_from_fresh_slot(slot)
+                    self._logger.info(
+                        f"Transaction {self.signature} reached confirmation status '{confirmation_status}' in slot {slot} after {time_taken} seconds")
+                    return
+            time.sleep(pause)
+
+        time_wasted_looking: float = time.time() - started_at
+        self._logger.warning(
+            f"Transaction {self.signature} disappeared despite spending {time_wasted_looking} seconds waiting for it")
+
+
 # # 🥭 RPCCaller class
 #
 # A `RPCCaller` extends the HTTPProvider with better error handling.
@@ -373,10 +429,9 @@ class RPCCaller(HTTPProvider):
         if len(params) > 1 and "commitment" in params[1] and params[1]["commitment"] == Processed:
             if "result" in response and isinstance(response["result"], Mapping) and "context" in response["result"] and isinstance(response["result"]["context"], Mapping) and "slot" in response["result"]["context"]:
                 slot: int = response["result"]["context"]["slot"]
-                self._logger.debug(f"{method}() data is from slot: {slot}")
                 if not self.slot_holder.is_acceptable(slot):
                     self._logger.warning(
-                        f"Result is from slot: {slot} - latest slot is: {self.slot_holder.latest_slot}")
+                        f"Result is from stale slot: {slot} - latest slot is: {self.slot_holder.latest_slot}")
                     raise StaleSlotException(self.name, self.cluster_url, self.slot_holder.latest_slot, slot)
 
         if "error" in response:
@@ -415,7 +470,10 @@ class RPCCaller(HTTPProvider):
                                            error_err, error_logs, self.instruction_reporter)
 
         if method == "getRecentBlockhash":
-            self._logger.debug(f"Recent blockhash fetched: {response}")
+            if "result" in response and "value" in response["result"] and "blockhash" in response["result"]["value"] and "context" in response["result"] and "slot" in response["result"]["context"]:
+                fresh_blockhash = Blockhash(response["result"]["value"]["blockhash"])
+                fresh_blockhash_slot = Blockhash(response["result"]["context"]["slot"])
+                self._logger.debug(f"Recent blockhash [slot: {fresh_blockhash_slot}]: {fresh_blockhash}")
 
         # The call succeeded.
         return typing.cast(RPCResponse, response)
@@ -474,6 +532,7 @@ class CompoundRPCCaller(HTTPProvider):
                     self._logger.debug(f"Shifted provider - now using: {self.__providers[0]}")
                 return result
             except (requests.exceptions.HTTPError,
+                    requests.exceptions.ConnectionError,
                     RateLimitException,
                     NodeIsBehindException,
                     StaleSlotException,
@@ -481,6 +540,9 @@ class CompoundRPCCaller(HTTPProvider):
                 all_exceptions += [exception]
                 self._logger.info(f"Moving to next provider - {provider} gave {exception}")
 
+        if len(all_exceptions) == 1:
+            raise all_exceptions[0]
+
         raise CompoundException(self.name, all_exceptions)
 
     def is_connected(self) -> bool:
@@ -497,27 +559,6 @@ class CompoundRPCCaller(HTTPProvider):
         return f"{self}"
 
 
-# This is purely to pass `maxRetries`: 0 in the TxOpts. (solana-py doesn't currently support this but Solana does.)
-class _MaxRetriesZeroClient(Client):
-    def _send_raw_transaction_args(
-        self, txn: typing.Union[bytes, str], opts: TxOpts
-    ) -> typing.Tuple[RPCMethod, str, typing.Dict[str, typing.Union[bool, Commitment, str]]]:
-
-        if isinstance(txn, bytes):
-            txn = b64encode(txn).decode("utf-8")
-
-        return (
-            RPCMethod("sendTransaction"),
-            txn,
-            {
-                self._skip_preflight_key: opts.skip_preflight,
-                self._preflight_comm_key: opts.preflight_commitment,
-                self._encoding_key: "base64",
-                "maxRetries": 0  # type: ignore
-            },
-        )
-
-
 class BetterClient:
     def __init__(self, client: Client, name: str, cluster_name: str, commitment: Commitment, skip_preflight: bool, encoding: str, blockhash_cache_duration: int, rpc_caller: CompoundRPCCaller) -> None:
         self._logger: logging.Logger = logging.getLogger(self.__class__.__name__)
@@ -529,6 +570,7 @@ class BetterClient:
         self.encoding: str = encoding
         self.blockhash_cache_duration: int = blockhash_cache_duration
         self.rpc_caller: CompoundRPCCaller = rpc_caller
+        self.executor: Executor = ThreadPoolExecutor()
 
     @staticmethod
     def from_configuration(name: str, cluster_name: str, cluster_urls: typing.Sequence[str], commitment: Commitment, skip_preflight: bool, encoding: str, blockhash_cache_duration: int, stale_data_pauses_before_retry: typing.Sequence[float], instruction_reporter: InstructionReporter) -> "BetterClient":
@@ -543,7 +585,7 @@ class BetterClient:
         blockhash_cache: typing.Union[BlockhashCache, bool] = False
         if blockhash_cache_duration > 0:
             blockhash_cache = BlockhashCache(blockhash_cache_duration)
-        client: Client = _MaxRetriesZeroClient(cluster_url, commitment=commitment, blockhash_cache=blockhash_cache)
+        client: Client = Client(cluster_url, commitment=commitment, blockhash_cache=blockhash_cache)
         client._provider = provider
 
         def __on_provider_change() -> None:
@@ -660,19 +702,17 @@ class BetterClient:
 
                 proper_opts = TxOpts(preflight_commitment=proper_commitment,
                                      skip_confirmation=opts.skip_confirmation,
-                                     skip_preflight=proper_skip_preflight)
+                                     skip_preflight=proper_skip_preflight,
+                                     max_retries=0)
 
                 response = self.compatible_client.send_transaction(transaction, *signers, opts=proper_opts)
                 signature: str = str(response["result"])
                 self._logger.debug(f"Transaction signature: {signature}")
 
                 if signature != _STUB_TRANSACTION_SIGNATURE:
-                    transaction_status = self.compatible_client.get_signature_statuses([signature])
-                    if "result" in transaction_status and "context" in transaction_status["result"] and "slot" in transaction_status["result"]["context"]:
-                        slot: int = transaction_status["result"]["context"]["slot"]
-                        self.rpc_caller.current.require_data_from_fresh_slot(slot)
-                    else:
-                        self._logger.error(f"Could not get status for signature {signature}")
+                    tx_reporter: TransactionWatcher = TransactionWatcher(
+                        self.compatible_client, self.rpc_caller.current.slot_holder, signature)
+                    self.executor.submit(tx_reporter.report_on_transaction)
                 else:
                     self._logger.error("Could not get status for stub signature")
 
