@@ -515,7 +515,11 @@ PERP_MARKET_INFO = construct.Struct(
 #     pub msrm_vault: Pubkey,
 #     pub fees_vault: Pubkey,
 #
-#     pub padding: [u8; 32], // padding used for future expansions
+#     pub max_mango_accounts: u32, // limits maximum number of MangoAccounts.v1 (closeable) accounts
+#     pub num_mango_accounts: u32, // number of MangoAccounts.v1
+#
+#     pub padding: [u8; 24], // padding used for future expansions
+#
 # }
 # ```
 GROUP = construct.Struct(
@@ -535,7 +539,9 @@ GROUP = construct.Struct(
     "srm_vault" / PublicKeyAdapter(),
     "msrm_vault" / PublicKeyAdapter(),
     "fees_vault" / PublicKeyAdapter(),
-    construct.Padding(32)
+    "max_mango_accounts" / DecimalAdapter(4),
+    "num_mango_accounts" / DecimalAdapter(4),
+    construct.Padding(24)
 )
 
 # # 🥭 ROOT_BANK
@@ -677,8 +683,21 @@ PERP_ACCOUNT = construct.Struct(
 #     /// This account cannot do anything except go through `resolve_bankruptcy`
 #     pub is_bankrupt: bool,
 #     pub info: [u8; INFO_LEN],
+#
+#     /// Starts off as zero pubkey and points to the AdvancedOrders account
+#     pub advanced_orders_key: Pubkey,
+#
+#     /// Can this account be upgraded to v1 so it can be closed
+#     pub not_upgradable: bool,
+#
+#     // Alternative authority/signer of transactions for a mango account
+#     pub delegate: Pubkey,
+#
 #     /// padding for expansions
-#     pub padding: [u8; 70],
+#     /// Note: future expansion can also be just done via isolated PDAs
+#     /// which can be computed independently and dont need to be linked from
+#     /// this account
+#     pub padding: [u8; 5],
 # }
 # ```
 MANGO_ACCOUNT = construct.Struct(
@@ -697,9 +716,12 @@ MANGO_ACCOUNT = construct.Struct(
     "client_order_ids" / construct.Array(MAX_PERP_OPEN_ORDERS, DecimalAdapter()),
     "msrm_amount" / DecimalAdapter(),
     "being_liquidated" / DecimalAdapter(1),
-    "is_bankrupt" / DecimalAdapter(1),
+    "is_bankrupt" / construct.Flag,
     "info" / construct.PaddedString(32, "utf8"),
-    construct.Padding(70)
+    "advanced_orders" / PublicKeyAdapter(),
+    "not_upgradable" / construct.Flag,
+    "delegate" / PublicKeyAdapter(),
+    construct.Padding(5)
 )
 
 # # 🥭 LIQUIDITY_MINING_INFO
@@ -1232,56 +1254,6 @@ SERUM_INSTRUCTION_VARIANT_FINDER = construct.Struct(
     "variant" / construct.BytesInteger(4, swapped=True)
 )
 
-# /// Place an order on a perp market
-# /// Accounts expected by this instruction (6):
-# /// 0. `[]` mango_group_ai - TODO
-# /// 1. `[writable]` mango_account_ai - TODO
-# /// 2. `[signer]` owner_ai - TODO
-# /// 3. `[]` mango_cache_ai - TODO
-# /// 4. `[writable]` perp_market_ai - TODO
-# /// 5. `[writable]` bids_ai - TODO
-# /// 6. `[writable]` asks_ai - TODO
-# /// 7. `[writable]` event_queue_ai - TODO
-PLACE_PERP_ORDER = construct.Struct(
-    "variant" / construct.Const(12, construct.BytesInteger(4, swapped=True)),
-
-    "price" / SignedDecimalAdapter(),
-    "quantity" / SignedDecimalAdapter(),
-    "client_order_id" / DecimalAdapter(),
-    "side" / DecimalAdapter(1),  # { buy: 0, sell: 1 }
-    "order_type" / DecimalAdapter(1)  # { limit: 0, ioc: 1, postOnly: 2 }
-)
-
-# /// Initialize a Mango account for a user
-# ///
-# /// Accounts expected by this instruction (4):
-# ///
-# /// 0. `[]` mango_group_ai - MangoGroup that this mango account is for
-# /// 1. `[writable]` mango_account_ai - the mango account data
-# /// 2. `[signer]` owner_ai - Solana account of owner of the mango account
-# /// 3. `[]` rent_ai - Rent sysvar account
-# InitMangoAccount,
-INIT_MANGO_ACCOUNT = construct.Struct(
-    "variant" / construct.Const(1, construct.BytesInteger(4, swapped=True))
-)
-
-
-# Cancel a Perp order using it's ID and Side.
-#
-# 0. `[]` mangoGroupPk
-# 1. `[writable]` mangoAccountPk
-# 2. `[signer]` ownerPk
-# 3. `[writable]` perpMarketPk
-# 4. `[writable]` bidsPk
-# 5. `[writable]` asksPk
-# 6. `[writable]` eventQueuePk
-CANCEL_PERP_ORDER = construct.Struct(
-    "variant" / construct.Const(14, construct.BytesInteger(4, swapped=True)),
-
-    "order_id" / DecimalAdapter(16),
-    "invalid_id_ok" / construct.Flag
-)
-
 
 # /// Deposit funds into mango account
 # ///
@@ -1377,6 +1349,152 @@ PLACE_SPOT_ORDER = construct.Struct(
     'limit' / DecimalAdapter(2),  # 50
 )
 
+
+# /// Place an order on a perp market
+# /// Accounts expected by this instruction (6):
+# /// 0. `[]` mango_group_ai - TODO
+# /// 1. `[writable]` mango_account_ai - TODO
+# /// 2. `[signer]` owner_ai - TODO
+# /// 3. `[]` mango_cache_ai - TODO
+# /// 4. `[writable]` perp_market_ai - TODO
+# /// 5. `[writable]` bids_ai - TODO
+# /// 6. `[writable]` asks_ai - TODO
+# /// 7. `[writable]` event_queue_ai - TODO
+PLACE_PERP_ORDER = construct.Struct(
+    "variant" / construct.Const(12, construct.BytesInteger(4, swapped=True)),
+
+    "price" / SignedDecimalAdapter(),
+    "quantity" / SignedDecimalAdapter(),
+    "client_order_id" / DecimalAdapter(),
+    "side" / DecimalAdapter(1),  # { buy: 0, sell: 1 }
+    "order_type" / DecimalAdapter(1)  # { limit: 0, ioc: 1, postOnly: 2 }
+)
+
+
+# Cancel a Perp order using only it's client ID.
+#
+# 0. `[]` mangoGroupPk
+# 1. `[writable]` mangoAccountPk
+# 2. `[signer]` ownerPk
+# 3. `[writable]` perpMarketPk
+# 4. `[writable]` bidsPk
+# 5. `[writable]` asksPk
+# 6. `[writable]` eventQueuePk
+CANCEL_PERP_ORDER_BY_CLIENT_ID = construct.Struct(
+    "variant" / construct.Const(13, construct.BytesInteger(4, swapped=True)),
+
+    "client_order_id" / DecimalAdapter(),
+    "invalid_id_ok" / construct.Flag
+)
+
+
+# Cancel a Perp order using it's ID and Side.
+#
+# 0. `[]` mangoGroupPk
+# 1. `[writable]` mangoAccountPk
+# 2. `[signer]` ownerPk
+# 3. `[writable]` perpMarketPk
+# 4. `[writable]` bidsPk
+# 5. `[writable]` asksPk
+# 6. `[writable]` eventQueuePk
+CANCEL_PERP_ORDER = construct.Struct(
+    "variant" / construct.Const(14, construct.BytesInteger(4, swapped=True)),
+
+    "order_id" / DecimalAdapter(16),
+    "invalid_id_ok" / construct.Flag
+)
+
+
+# Run the Mango crank.
+#
+# 0. `[]` mangoGroupPk
+# 1. `[]` perpMarketPk
+# 2. `[writable]` eventQueuePk
+# 3+ `[writable]` mangoAccountPks...
+CONSUME_EVENTS = construct.Struct(
+    "variant" / construct.Const(15, construct.BytesInteger(4, swapped=True)),
+
+    "limit" / DecimalAdapter()
+)
+
+
+# /// Settle all funds from serum dex open orders
+# ///
+# /// Accounts expected by this instruction (18):
+# ///
+# /// 0. `[]` mango_group_ai - MangoGroup that this mango account is for
+# /// 1. `[]` mango_cache_ai - MangoCache for this MangoGroup
+# /// 2. `[signer]` owner_ai - MangoAccount owner
+# /// 3. `[writable]` mango_account_ai - MangoAccount
+# /// 4. `[]` dex_prog_ai - program id of serum dex
+# /// 5.  `[writable]` spot_market_ai - dex MarketState account
+# /// 6.  `[writable]` open_orders_ai - open orders for this market for this MangoAccount
+# /// 7. `[]` signer_ai - MangoGroup signer key
+# /// 8. `[writable]` dex_base_ai - base vault for dex MarketState
+# /// 9. `[writable]` dex_quote_ai - quote vault for dex MarketState
+# /// 10. `[]` base_root_bank_ai - MangoGroup base vault acc
+# /// 11. `[writable]` base_node_bank_ai - MangoGroup quote vault acc
+# /// 12. `[]` quote_root_bank_ai - MangoGroup quote vault acc
+# /// 13. `[writable]` quote_node_bank_ai - MangoGroup quote vault acc
+# /// 14. `[writable]` base_vault_ai - MangoGroup base vault acc
+# /// 15. `[writable]` quote_vault_ai - MangoGroup quote vault acc
+# /// 16. `[]` dex_signer_ai - dex Market signer account
+# /// 17. `[]` spl token program
+SETTLE_FUNDS = construct.Struct(
+    "variant" / construct.Const(19, construct.BytesInteger(4, swapped=True))
+)
+
+
+# /// Cancel an order using dex instruction
+# ///
+# /// Accounts expected by this instruction ():
+# ///
+# CancelSpotOrder {
+#     order: serum_dex::instruction::CancelOrderInstructionV2,
+# },
+CANCEL_SPOT_ORDER = construct.Struct(
+    "variant" / construct.Const(20, construct.BytesInteger(4, swapped=True)),
+
+    'side' / DecimalAdapter(4),
+    "order_id" / DecimalAdapter(16)
+)
+
+
+# /// Redeem the mngo_accrued in a PerpAccount for MNGO in MangoAccount deposits
+# ///
+# /// Accounts expected by this instruction (11):
+# /// 0. `[]` mango_group_ai - MangoGroup that this mango account is for
+# /// 1. `[]` mango_cache_ai - MangoCache
+# /// 2. `[writable]` mango_account_ai - MangoAccount
+# /// 3. `[signer]` owner_ai - MangoAccount owner
+# /// 4. `[]` perp_market_ai - PerpMarket
+# /// 5. `[writable]` mngo_perp_vault_ai
+# /// 6. `[]` mngo_root_bank_ai
+# /// 7. `[writable]` mngo_node_bank_ai
+# /// 8. `[writable]` mngo_bank_vault_ai
+# /// 9. `[]` signer_ai - Group Signer Account
+# /// 10. `[]` token_prog_ai - SPL Token program id
+REDEEM_MNGO = construct.Struct(
+    "variant" / construct.Const(33, construct.BytesInteger(4, swapped=True))
+)
+
+
+# /// Cancel all perp open orders (batch cancel)
+# ///
+# /// Accounts expected: 6
+# /// 0. `[]` mango_group_ai - MangoGroup
+# /// 1. `[writable]` mango_account_ai - MangoAccount
+# /// 2. `[signer]` owner_ai - Owner of Mango Account
+# /// 3. `[writable]` perp_market_ai - PerpMarket
+# /// 4. `[writable]` bids_ai - Bids acc
+# /// 5. `[writable]` asks_ai - Asks acc
+CANCEL_ALL_PERP_ORDERS = construct.Struct(
+    "variant" / construct.Const(39, construct.BytesInteger(4, swapped=True)),
+
+    "limit" / DecimalAdapter(1)
+)
+
+
 # Seems identical to PLACE_SPOT_ORDER except for variant.
 # { isSigner: false, isWritable: false, pubkey: mangoGroupPk },
 # { isSigner: false, isWritable: true, pubkey: mangoAccountPk },
@@ -1418,76 +1536,48 @@ PLACE_SPOT_ORDER_2 = construct.Struct(
     'limit' / DecimalAdapter(2),  # 50
 )
 
-# /// Cancel an order using dex instruction
-# ///
-# /// Accounts expected by this instruction ():
-# ///
-# CancelSpotOrder {
-#     order: serum_dex::instruction::CancelOrderInstructionV2,
-# },
-CANCEL_SPOT_ORDER = construct.Struct(
-    "variant" / construct.Const(20, construct.BytesInteger(4, swapped=True)),
 
-    'side' / DecimalAdapter(4),
-    "order_id" / DecimalAdapter(16)
+# /// Delete a mango account and return lamports
+# ///
+# /// Accounts expected by this instruction (3):
+# ///
+# /// 0. `[writable]` mango_group_ai - MangoGroup that this mango account is for
+# /// 1. `[writable]` mango_account_ai - the mango account data
+# /// 2. `[signer]` owner_ai - Solana account of owner of the mango account
+CLOSE_MANGO_ACCOUNT = construct.Struct(
+    "variant" / construct.Const(50, construct.BytesInteger(4, swapped=True))
 )
 
 
-# Cancel a Perp order using only it's client ID.
-#
-# 0. `[]` mangoGroupPk
-# 1. `[writable]` mangoAccountPk
-# 2. `[signer]` ownerPk
-# 3. `[writable]` perpMarketPk
-# 4. `[writable]` bidsPk
-# 5. `[writable]` asksPk
-# 6. `[writable]` eventQueuePk
-CANCEL_PERP_ORDER_BY_CLIENT_ID = construct.Struct(
-    "variant" / construct.Const(13, construct.BytesInteger(4, swapped=True)),
-
-    "client_order_id" / DecimalAdapter(),
-    "invalid_id_ok" / construct.Flag
-)
-
-# Run the Mango crank.
-#
-# 0. `[]` mangoGroupPk
-# 1. `[]` perpMarketPk
-# 2. `[writable]` eventQueuePk
-# 3+ `[writable]` mangoAccountPks...
-CONSUME_EVENTS = construct.Struct(
-    "variant" / construct.Const(15, construct.BytesInteger(4, swapped=True)),
-
-    "limit" / DecimalAdapter()
-)
-
-# /// Settle all funds from serum dex open orders
+# /// Create a PDA mango account for a user
 # ///
-# /// Accounts expected by this instruction (18):
+# /// Accounts expected by this instruction (4):
 # ///
-# /// 0. `[]` mango_group_ai - MangoGroup that this mango account is for
-# /// 1. `[]` mango_cache_ai - MangoCache for this MangoGroup
-# /// 2. `[signer]` owner_ai - MangoAccount owner
-# /// 3. `[writable]` mango_account_ai - MangoAccount
-# /// 4. `[]` dex_prog_ai - program id of serum dex
-# /// 5.  `[writable]` spot_market_ai - dex MarketState account
-# /// 6.  `[writable]` open_orders_ai - open orders for this market for this MangoAccount
-# /// 7. `[]` signer_ai - MangoGroup signer key
-# /// 8. `[writable]` dex_base_ai - base vault for dex MarketState
-# /// 9. `[writable]` dex_quote_ai - quote vault for dex MarketState
-# /// 10. `[]` base_root_bank_ai - MangoGroup base vault acc
-# /// 11. `[writable]` base_node_bank_ai - MangoGroup quote vault acc
-# /// 12. `[]` quote_root_bank_ai - MangoGroup quote vault acc
-# /// 13. `[writable]` quote_node_bank_ai - MangoGroup quote vault acc
-# /// 14. `[writable]` base_vault_ai - MangoGroup base vault acc
-# /// 15. `[writable]` quote_vault_ai - MangoGroup quote vault acc
-# /// 16. `[]` dex_signer_ai - dex Market signer account
-# /// 17. `[]` spl token program
-SETTLE_FUNDS = construct.Struct(
-    "variant" / construct.Const(19, construct.BytesInteger(4, swapped=True))
+# /// 0. `[writable]` mango_group_ai - MangoGroup that this mango account is for
+# /// 1. `[writable]` mango_account_ai - the mango account data
+# /// 2. `[signer]` owner_ai - Solana account of owner of the mango account
+# /// 3. `[]` system_prog_ai - System program
+CREATE_MANGO_ACCOUNT = construct.Struct(
+    "variant" / construct.Const(55, construct.BytesInteger(4, swapped=True)),
+    "account_num" / DecimalAdapter()
 )
 
-# /// Initialize open orders
+
+# /// https://github.com/blockworks-foundation/mango-v3/pull/97/
+# /// Set delegate authority to mango account which can do everything regular account can do
+# /// except Withdraw and CloseMangoAccount. Set to Pubkey::default() to revoke delegate
+# ///
+# /// Accounts expected: 4
+# /// 0. `[]` mango_group_ai - MangoGroup
+# /// 1. `[writable]` mango_account_ai - MangoAccount
+# /// 2. `[signer]` owner_ai - Owner of Mango Account
+# /// 3. `[]` delegate_ai - delegate
+SET_DELEGATE = construct.Struct(
+    "variant" / construct.Const(58, construct.BytesInteger(4, swapped=True))
+)
+
+
+# /// Create an OpenOrders PDA and initialize it with InitOpenOrders call to serum dex
 # ///
 # /// Accounts expected by this instruction (8):
 # ///
@@ -1495,39 +1585,23 @@ SETTLE_FUNDS = construct.Struct(
 # /// 1. `[writable]` mango_account_ai - MangoAccount
 # /// 2. `[signer]` owner_ai - MangoAccount owner
 # /// 3. `[]` dex_prog_ai - program id of serum dex
-# /// 4. `[writable]` open_orders_ai - open orders for this market for this MangoAccount
+# /// 4. `[writable]` open_orders_ai - open orders PDA
 # /// 5. `[]` spot_market_ai - dex MarketState account
 # /// 6. `[]` signer_ai - Group Signer Account
-# /// 7. `[]` rent_ai - Rent sysvar account
-INIT_SPOT_OPEN_ORDERS = construct.Struct(
-    "variant" / construct.Const(32, construct.BytesInteger(4, swapped=True))
+# /// 7. `[]` system_prog_ai - System program
+CREATE_SPOT_OPEN_ORDERS = construct.Struct(
+    "variant" / construct.Const(60, construct.BytesInteger(4, swapped=True))
 )
 
-# /// Redeem the mngo_accrued in a PerpAccount for MNGO in MangoAccount deposits
-# ///
-# /// Accounts expected by this instruction (11):
-# /// 0. `[]` mango_group_ai - MangoGroup that this mango account is for
-# /// 1. `[]` mango_cache_ai - MangoCache
-# /// 2. `[writable]` mango_account_ai - MangoAccount
-# /// 3. `[signer]` owner_ai - MangoAccount owner
-# /// 4. `[]` perp_market_ai - PerpMarket
-# /// 5. `[writable]` mngo_perp_vault_ai
-# /// 6. `[]` mngo_root_bank_ai
-# /// 7. `[writable]` mngo_node_bank_ai
-# /// 8. `[writable]` mngo_bank_vault_ai
-# /// 9. `[]` signer_ai - Group Signer Account
-# /// 10. `[]` token_prog_ai - SPL Token program id
-REDEEM_MNGO = construct.Struct(
-    "variant" / construct.Const(33, construct.BytesInteger(4, swapped=True))
-)
 
 UNSPECIFIED = construct.Struct(
     "variant" / DecimalAdapter(4)
 )
 
+
 InstructionParsersByVariant = {
     0: UNSPECIFIED,  # INIT_MANGO_GROUP,
-    1: INIT_MANGO_ACCOUNT,  # INIT_MANGO_ACCOUNT,
+    1: UNSPECIFIED,  # INIT_MANGO_ACCOUNT,
     2: DEPOSIT,  # DEPOSIT,
     3: WITHDRAW,  # WITHDRAW,
     4: UNSPECIFIED,  # ADD_SPOT_MARKET,
@@ -1558,14 +1632,14 @@ InstructionParsersByVariant = {
     29: UNSPECIFIED,  # SETTLE_FEES,
     30: UNSPECIFIED,  # RESOLVE_PERP_BANKRUPTCY,
     31: UNSPECIFIED,  # RESOLVE_TOKEN_BANKRUPTCY,
-    32: INIT_SPOT_OPEN_ORDERS,  # INIT_SPOT_OPEN_ORDERS,
+    32: UNSPECIFIED,  # INIT_SPOT_OPEN_ORDERS,
     33: REDEEM_MNGO,  # REDEEM_MNGO,
     34: UNSPECIFIED,  # ADD_MANGO_ACCOUNT_INFO,
     35: UNSPECIFIED,  # DEPOSIT_MSRM,
     36: UNSPECIFIED,  # WITHDRAW_MSRM,
     37: UNSPECIFIED,  # CHANGE_PERP_MARKET_PARAMS,
     38: UNSPECIFIED,  # SET_GROUP_ADMIN,
-    39: UNSPECIFIED,  # CANCEL_ALL_PERP_ORDERS,
+    39: CANCEL_ALL_PERP_ORDERS,  # CANCEL_ALL_PERP_ORDERS,
     40: UNSPECIFIED,  # FORCE_SETTLE_QUOTE_POSITIONS,
     41: PLACE_SPOT_ORDER_2,  # PLACE_SPOT_ORDER_2,
     42: UNSPECIFIED,  # INIT_ADVANCED_ORDERS,
@@ -1575,4 +1649,16 @@ InstructionParsersByVariant = {
     46: UNSPECIFIED,  # CREATE_PERP_MARKET,
     47: UNSPECIFIED,  # CHANGE_PERP_MARKET_PARAMS2,
     48: UNSPECIFIED,  # UPDATE_MARGIN_BASKET,
+    49: UNSPECIFIED,  # CHANG_MAX_MANGO_ACCOUNTS,
+    50: CLOSE_MANGO_ACCOUNT,  # CLOSE_MANGO_ACCOUNT,
+    51: UNSPECIFIED,  # CLOSE_SPOT_OPEN_ORDERS,
+    52: UNSPECIFIED,  # CLOSE_ADVANCED_ORDERS,
+    53: UNSPECIFIED,  # CREATE_DUST_ACCOUNT,
+    54: UNSPECIFIED,  # RESOLVE_DUST,
+    55: CREATE_MANGO_ACCOUNT,  # CREATE_MANGO_ACCOUNT,
+    56: UNSPECIFIED,  # UPGRADE_MANGO_ACCOUNT_V0_V1,
+    57: UNSPECIFIED,  # CANCEL_PERP_ORDER_SIDE,
+    58: SET_DELEGATE,  # SET_DELEGATE,
+    59: UNSPECIFIED,  # CHANGE_SPOT_MARKET_PARAMS,
+    60: CREATE_SPOT_OPEN_ORDERS,  # CREATE_SPOT_OPEN_ORDERS,
 }
